unit uLoggerSingleton;

interface

type
  TLoggerSingleton = class
  private
    // varivel que aponta para o arquivo de log
    ArquivoLog: TextFile;

    // o construtor  declarado como privado
    // para evitar que seja chamado por outras classes
    constructor Create;
  public
    // mtodo principal do Singleton
    class function ObterInstancia: TLoggerSingleton;

    class function NewInstance: TObject; override;

    // mtodo para registrar o texto do parmetro no arquivo de log
    procedure RegistrarLog(const Texto: string);

    destructor Destroy; override;
  end;

var
  Instancia: TLoggerSingleton;

implementation

uses
  Forms, SysUtils;

{ TLogger }

constructor TLoggerSingleton.Create;
var
  DiretorioAplicacao: string;
begin
  // associa o aquivo "Log.txt" que est na pasta do projeto
  DiretorioAplicacao := ExtractFilePath(Application.ExeName);
  AssignFile(ArquivoLog, DiretorioAplicacao + 'Log.txt');

  // se o arquivo no existir,  criado
  if not FileExists(DiretorioAplicacao + 'Log.txt') then
  begin
    Rewrite(ArquivoLog);
    CloseFile(ArquivoLog);
  end;
end;

destructor TLoggerSingleton.Destroy;
begin
  // libera o Singleton da memria
  FreeAndNil(Instancia);

  inherited;
end;

class function TLoggerSingleton.NewInstance: TObject;
begin
  // se j houver uma instncia, ela  retornada
  // caso contrrio, o objeto  instanciado antes de ser retornado
 
  if not Assigned(Instancia) then
  begin
    // chama a funo "NewInstance" da herana (TObject)
    Instancia := TLoggerSingleton(inherited NewInstance);
  end;
 
  result := Instancia;
end;

class function TLoggerSingleton.ObterInstancia: TLoggerSingleton;
begin
  // chama o mtodo Create, que cria (uma nica vez) e retorna a instncia
  result := TLoggerSingleton.Create;
end;

procedure TLoggerSingleton.RegistrarLog(const Texto: string);
var
  sDataHora: string;
begin
  // abre o arquivo de log para edio
  Append(ArquivoLog);

  // escreve a data, hora e texto no arquivo de log
  sDataHora := FormatDateTime('[dd/mm/yyyy hh:nn:ss] ', Now);
  WriteLn(ArquivoLog, sDataHora + Texto);

  // fecha o arquivo
  CloseFile(ArquivoLog);
end;

end.
